/*
 *  Copyright (C) 2001 Philip Langdale
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2, or (at your option)
 *  any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "galeon.h"
#include "ContentHandler.h"
#include "nsCOMPtr.h"
#include "nsIContentHandler.h"
#include "nsISupportsArray.h"
#include "nsIURI.h"
#include "nsIChannel.h"
#include "nsIFactory.h"
#include "nsString.h"
#include "nsIServiceManager.h"
#include "nsIUnicodeEncoder.h"
#include "nsIRequestObserver.h"


/**
 * class GContentHandler: 
 *
 */
class GContentHandler: public nsIContentHandler
{
	public:
		GContentHandler ();
  		virtual ~GContentHandler();

		NS_DECL_ISUPPORTS
		NS_DECL_NSICONTENTHANDLER
  
};

NS_IMPL_ISUPPORTS1 (GContentHandler, nsIContentHandler)

/**
 * GContentHandler::GContentHandler: constructor
 */
GContentHandler::GContentHandler ()
{
	NS_INIT_REFCNT();
}

/**
 * GContentHandler::~GContentHandler: destructor
 */
GContentHandler::~GContentHandler ()
{
}

NS_IMETHODIMP GContentHandler::HandleContent (const char * aContentType,
					      const char * aCommand,
					      const char * aWindowTarget,
					      nsISupports * aWindowContext,
					      nsIRequest *aRequest)
{
	nsresult rv = NS_OK;
	if (!aRequest)
		return NS_ERROR_NULL_POINTER;
  	// First of all, get the content type and make sure it is a 
  	// content type we know how to handle!
//	if (nsCRT::strcasecmp(aContentType, "application/zip") == 0)
//	{
		nsCOMPtr<nsIURI> aUri;
		nsCOMPtr<nsIChannel> aChannel = do_QueryInterface(aRequest);
		if(!aChannel) return NS_ERROR_FAILURE;
 
		rv = aChannel->GetURI(getter_AddRefs(aUri));
		aRequest->Cancel(NS_BINDING_ABORTED);
		if (aUri)
		{
			gchar *gUri;
			aUri->GetSpec (&gUri);
		       	save_url(gUri);
			g_free (gUri);
		}
//	}
	return rv;
}

/**
 * CPromptServiceFactory: factory class
 */ 
class GContentHandlerFactory : public nsIFactory
{
	public:
		NS_DECL_ISUPPORTS
		NS_DECL_NSIFACTORY

		GContentHandlerFactory();
		virtual ~GContentHandlerFactory();
};

NS_IMPL_ISUPPORTS1(GContentHandlerFactory, nsIFactory)

/**
 * GContentHandlerFactory::GContentHandlerFactory: factory class constructor
 */ 
GContentHandlerFactory::GContentHandlerFactory()
{
	NS_INIT_ISUPPORTS();
}

/**
 * GContentHandlerFactory::~GContentHandlerFactory: factory class destructor
 */ 
GContentHandlerFactory::~GContentHandlerFactory()
{
}

/**
 * GContentHandlerFactory::CreateInstance: create a single factory
 */ 
NS_IMETHODIMP GContentHandlerFactory::CreateInstance(nsISupports *aOuter, 
						     const nsIID & aIID, 
						     void **aResult)
{
	NS_ENSURE_ARG_POINTER(aResult);
  
	*aResult = NULL;  
	GContentHandler *inst = new GContentHandler;    
	if (inst == NULL)
	{
		return NS_ERROR_OUT_OF_MEMORY;
	}
    
	nsresult rv = inst->QueryInterface(aIID, aResult);
	if (rv != NS_OK)
	{
		delete inst;  
	}  

	return rv;
}

/**
 * GContentHandlerFactory::LockFactory:
 */ 
NS_IMETHODIMP GContentHandlerFactory::LockFactory(PRBool lock)
{
	return NS_OK;
}

/**
 * NS_NewContentHandlerFactory:
 */ 
nsresult NS_NewContentHandlerFactory(nsIFactory** aFactory)
{
	NS_ENSURE_ARG_POINTER(aFactory);
	*aFactory = nsnull;

	GContentHandlerFactory *result = new GContentHandlerFactory;
	if (result == NULL)
	{
		return NS_ERROR_OUT_OF_MEMORY;
	}
    
	NS_ADDREF(result);
	*aFactory = result;

	return NS_OK;
}
